/****************************************************************************
 Module
   RackEventChecker.c

 Author
   Lauren Taylor
****************************************************************************/

// this will pull in the symbolic definitions for events, which we will want
// to post in response to detecting events
#include "ES_Configure.h"
// This gets us the prototype for ES_PostAll
#include "ES_Framework.h"
// this will get us the structure definition for events, which we will need
// in order to post events in response to detecting events
#include "ES_Events.h"
// if you want to use distribution lists then you need those function
// definitions too.
#include "ES_PostList.h"
// This include will pull in all of the headers from the service modules
// providing the prototypes for all of the post functions
#include "ES_ServiceHeaders.h"
// this test harness for the framework references the serial routines that
// are defined in ES_Port.c
#include "ES_Port.h"
// include our own prototypes to insure consistency between header &
// actual functions definition
#include "RackEventChecker.h"
#include "LauncherService.h"
#include "PIC32PortHAL.h"
#include "terminal.h"
#include "dbprintf.h"

/*----------------------------- Module Defines ----------------------------*/

#define LIMITSWITCH_PORT _Port_B
#define LIMITSWITCH_PIN _Pin_4 // Pin 11

/*---------------------------- Module Variables ---------------------------*/

static bool RackPulled = false;

/*------------------------------ Module Code ------------------------------*/

void InitRackStatus(void) {
    PortSetup_ConfigureDigitalInputs(LIMITSWITCH_PORT, LIMITSWITCH_PIN);
}

/****************************************************************************
 Function
   CheckRackSwitch
 Parameters
   None
 Returns
   bool: true if limit switch has been pressed and then decompressed
 Description
   Checks for rack to deploy limit switch, then checks for limit switch to
   raise again to indicate that shoot has occurred
 Author
    Lauren Taylor
****************************************************************************/
bool CheckRackSwitch(void)
{
  if (PORTBbits.RB4 == 0) { // Rack has been pulled
    RackPulled = true;
  }
  if (PORTBbits.RB4 == 1 && RackPulled == true) {
    ES_Event_t ThisEvent;
    ThisEvent.EventType = ES_SHOOT_COMPLETE;
    PostLauncherService(ThisEvent);
    RackPulled = false;
    return true;
  }
  return false;
}

